/***************************************************************************
 *                                                                         *
 *  MAXIMUS-CBCS Source Code, Version 1.02                                 *
 *  Copyright 1989, 1990 by Scott J. Dudley.  All rights reserved.         *
 *                                                                         *
 *  Message area routines (replying, changing, nodelist stuff,             *
 *  I)nquire, shell for local/MaxEd/BORED.                                 *
 *                                                                         *
 *  For complete details of the licensing restrictions, please refer to    *
 *  the licence agreement, which is published in its entirety in MAX.C     *
 *  and LICENCE.MAX.                                                       *
 *                                                                         *
 *  USE OF THIS FILE IS SUBJECT TO THE RESTRICTIONS CONTAINED IN THE       *
 *  MAXIMUS-CBCS LICENSING AGREEMENT.  IF YOU DO NOT FIND THE TEXT OF THIS *
 *  AGREEMENT IN ANY OF THE  AFOREMENTIONED FILES, OR IF YOU DO NOT HAVE   *
 *  THESE FILES, YOU SHOULD IMMEDIATELY CONTACT THE AUTHOR AT ONE OF THE   *
 *  ADDRESSES LISTED BELOW.  IN NO EVENT SHOULD YOU PROCEED TO USE THIS    *
 *  FILE WITHOUT HAVING ACCEPTED THE TERMS OF THE MAXIMUS-CBCS LICENSING   *
 *  AGREEMENT, OR SUCH OTHER AGREEMENT AS YOU ARE ABLE TO REACH WITH THE   *
 *  AUTHOR.                                                                *
 *                                                                         *
 *  You can contact the author at one of the address listed below:         *
 *                                                                         *
 *  Scott Dudley           FidoNet  1:249/106                              *
 *  777 Downing St.        IMEXnet  89:483/202                             *
 *  Kingston, Ont.         Internet f106.n249.z1.fidonet.org               *
 *  Canada - K7M 5N3       BBS      (613) 389-8315 - HST/14.4K, 24hrs      *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include "prog.h"
#include "max.h"
#include "max_msg.h"
#include "rep.h"


static int pascal Msg_Change_Err(void);


int pascal NetList(void)
{
  struct _ndi ndi[NUM_NDI];
  struct _newnode nodep;

  long n6size;

  int idxfile,
      nlfile,
      offset,
      x,y,
      ret;

  char temp[PATHLEN],
       nonstop;

  display_line=display_col=1;
  ret=1;
  nonstop=FALSE;

  sprintf(temp,idxnode,PRM(net_info));

  if ((idxfile=shopen(temp,O_RDONLY | O_BINARY))==-1)
  {
    logit(cantopen,temp);
    return 0;
  }

  if (prm.nlver==6)
  {
    sprintf(temp,idxnode,PRM(net_info));

    /* This finds number of nodes in IDX/DAT */
    x=((int)fsize(temp))/sizeof(struct _ndi);

    if (!x)
      return 0;

    sprintf(temp,datnode,PRM(net_info));

    n6size=fsize(temp);     /* Calculate the dynamic V6 size by dividing   */
    n6size=(long)n6size/x;  /* total .DAT size by number of nodes          */
  }
  else sprintf(temp,sysnode,PRM(net_info));

  if ((nlfile=shopen(temp,O_RDONLY | O_BINARY))==-1)
  {
    logit(cantopen,temp);
    return 0;
  }

  for (offset=0;! eof(idxfile);offset += NUM_NDI)
  {
    y=read(idxfile,(char *)ndi,sizeof(struct _ndi)*NUM_NDI) / sizeof(struct _ndi);

    for (x=0;x < y;x++)
    {
      if (ndi[x].node <= 0)
      {
        lseek(nlfile,((long)offset+x)*(prm.nlver==6 ? (long)n6size :
              (long)sizeof(struct _node)),SEEK_SET);

        if (read(nlfile,(char *)&nodep,prm.nlver==6 ? sizeof(struct _newnode) :
             sizeof(struct _node)) <= 0)
        {
          close(nlfile);
          logit(cantread,temp);
          return 0;
        }

        if (ndi[x].node==-2)
          Puts(pad_zone);
        else if (ndi[x].node==-1)
          Puts(pad_region);
        else Puts(pad_net);

        Printf(node_listing,ndi[x].net,prm.nlver==6 ?
               nodep.SystemName :
               (*(struct _node *)(&nodep)).name,
               prm.nlver==6 ? nodep.MiscInfo :
               (*(struct _node *)(&nodep)).city);

        if (MoreYnBreak(&nonstop,CYAN))
        {
          ret=0;
          goto EndLoop;
        }
      }
    }
  }

EndLoop:

  close(nlfile);
  close(idxfile);

  return ret;
}


int pascal NodeList(int zone,int net_number)
{
  struct _ndi ndi[NUM_NDI];
  struct _newnode nodep;

  long n6size;

  int current_zone,
      idxfile,
      nlfile,
      offset,
      x,y,
      ret;

  char temp[PATHLEN],
       nonstop;

  display_line=display_col=1;
  current_zone=1;
  ret=1;

  nonstop=FALSE;

  sprintf(temp,idxnode,PRM(net_info));

  if ((idxfile=shopen(temp,O_RDONLY | O_BINARY))==-1)
  {
    logit(cantopen,temp);
    return 0;
  }

  if (prm.nlver==6)
  {
    sprintf(temp,idxnode,PRM(net_info));

    /* This finds number of nodes in IDX/DAT */
    x=((int)fsize(temp))/sizeof(struct _ndi);

    if (!x)
      return 0;

    sprintf(temp,datnode,PRM(net_info));

    n6size=fsize(temp);     /* Calculate the dynamic V6 size by dividing   */
    n6size=(long)n6size/x;  /* total .DAT size by number of nodes          */
  }
  else sprintf(temp,sysnode,PRM(net_info));

  if ((nlfile=shopen(temp,O_RDONLY | O_BINARY))==-1)
  {
    logit(cantopen,temp);
    return 0;
  }

  for (offset=0;! eof(idxfile);offset += NUM_NDI)
  {
    y=read(idxfile,(char *)ndi,sizeof(struct _ndi)*NUM_NDI) / sizeof(struct _ndi);

    for (x=0;x < y;x++)
    {
      if (ndi[x].node==-2)
        current_zone=ndi[x].net;

      if ((zone==0 || zone==current_zone || prm.nlver==5) &&
          ndi[x].net==net_number)
      {
        lseek(nlfile,((long)offset+x)*(prm.nlver==6 ? (long)n6size :
              (long)sizeof(struct _node)),SEEK_SET);

        if (read(nlfile,(char *)&nodep,prm.nlver==6 ? sizeof(struct _newnode) :
             sizeof(struct _node)) <= 0)
        {
          close(nlfile);
          logit(cantread,temp);
          return 0;
        }

        if (ndi[x].node==-2)
          Puts(pad_zone);
        else if (ndi[x].node==-1)
          Puts(pad_region);
        else if (ndi[x].node==0)
          Puts(pad_net);
        else Puts(pad_none);

        Printf(node_listing,ndi[x].node > 0 ? ndi[x].node : ndi[x].net,
               prm.nlver==6 ? nodep.SystemName :
               (*(struct _node *)(&nodep)).name,
               prm.nlver==6 ? nodep.MiscInfo :
               (*(struct _node *)(&nodep)).city);

        if (MoreYnBreak(&nonstop,CYAN))
        {
          ret=0;
          goto EndLoop;
        }
      }
    }
  }

EndLoop:

  close(nlfile);
  close(idxfile);

  return ret;
}



void pascal Msg_Inquire(void)
{
  struct _msg msg;
  struct _rep rexp;

  char temp[PATHLEN],
       nonstop,             /* If we're doing non-stop listing   */
       from[45],            /* Holding area for FROM             */
       to[45],              /* Holding area for TO               */
       subject[81],         /* Holding area for SUBJ             */
       expr[PATHLEN],
       *p;

  int msgfile,
      found,
      x,y,z;

  while (1)
  {
    WhiteN();

    if (hotkeys(usr))
      OpusGetsLL(temp,PATHLEN,s(inq_prompt));
    else
    {
      if (isnl(linebuf))
        Puts(s(inq_p1));

      OpusGetsLL(temp,PATHLEN,s(inq_p2));
    }

    strcpy(expr,temp);

    if (eqstr(temp,qmark))
      Display_File(0,NULL,PRM(msg_inquire));
    else break;
  }

  strlwr(temp);

  if (isnl(temp) || Compile_REP(temp,&rexp) != 0)
    Puts(s(inq_badexpr));
  else
  {
    display_line=display_col=1;
    nonstop=FALSE;

    Printf(s(inq_search),temp);

    for (found=FALSE,x=num_msg-1;x >= 0;x--)
    {
      if (MoreYnBreak(&nonstop,CYAN))
        break;

      if (brk_trapped || mdm_halt())
      {
        brk_trapped=0;
        mdm_dump(DUMP_all);
        break;
      }

      sprintf(temp,d_msg,msgs[x]);

      if ((msgfile=shopen(temp,O_RDONLY | O_BINARY)) != -1)
      {
        read(msgfile,(char *)&msg,sizeof(struct _msg));
        close(msgfile);

        if (CanSeeMsg(&msg))
        {
          strcpy(to,strlwr(msg.to));
          strcpy(from,strlwr(msg.from));
          strcpy(subject,strlwr(msg.subj));

          Strip_Ansi(to,usr.msg,msgs[x]);
          Strip_Ansi(from,usr.msg,msgs[x]);
          Strip_Ansi(subject,usr.msg,msgs[x]);

          if ((p=stristrx(to,&rexp)) != NULL)
            z=1;
          else
          {
            if ((p=stristrx(from,&rexp)) != NULL)
              z=2;
            else
            {
              p=stristrx(subject,&rexp);
              z=3;
            }
          }

          if (p)  /* We found something! */
          {
            found=TRUE;

            if (*p)
            {
              /* Move everything AFTER the string we found, to the right
                 three places, so we can insert the gray colour code here */

              memmove(p+rexp.max_ch+3,p+rexp.max_ch,strlen(p+rexp.max_ch)+1);
              p[rexp.max_ch  ]='\x16';
              p[rexp.max_ch+1]='\x01';
              p[rexp.max_ch+2]='\x07';

              for (y=0;y < rexp.max_ch;y++)
                p[y]=toupper(p[y]);

              /* Now move everything after the FRONT of the string we found,
                 to the right three places, so we can insert the light-yellow
                 colour code.                                              */

              memmove(p+3,p,strlen(p)+1);
              p[0]='\x16';
              p[1]='\x01';
              p[2]='\x0e';
            }

            /* Now fancy_str anything in the fields that we didn't touch */

            switch (z)
            {
              case 1:             /* String found in TO: field */
                fancier_str(from);
                fancy_str(subject);
                break;

              case 2:             /* String found in the FROM: field */
                fancy_str(subject);
                fancier_str(to);
                break;

              case 3:             /* String found in the SUBJ: field */
                fancier_str(from);
                fancier_str(to);
                break;
            }

            /* Now display the message header */

            Printf(s(ifmt_1),msgs[x],to);

            if (MoreYnBreak(&nonstop,CYAN))
              break;

            Printf(s(ifmt_2),from);

            if (MoreYnBreak(&nonstop,CYAN))
              break;

            Printf(s(ifmt_3),subject);

            if (MoreYnBreak(&nonstop,CYAN))
              break;

            Putc('\n');
          }
        }
        else Mdm_check(); /* Make sure user didn't go away */
      }
    }

    /* Free set-search tables */

    for (x=0;x < rexp.max_ch;x++)
      if (rexp.table[x])
        free(rexp.table[x]);

    if (!found)
      Printf(s(inq_nf),expr);
  }

  Clear_KBuffer();
}







int pascal Msg_Change(void)
{
  struct _msg msg,
              hwmsg;

  char temp[PATHLEN];

  int msgfile,
      aborted,
      m_zone,
      m_net,
      m_node,
      m_point,
      hwmf,
      hwm,
      x;
    
  lmsg=(*linebuf ? atoi(linebuf) : last_msg);
  *linebuf='\0';
  
  sprintf(temp,d_msg,lmsg);

  if ((msgfile=shopen(temp,O_RDWR | O_BINARY)) != -1 &&
      read(msgfile,(char *)&msg,sizeof(struct _msg))==sizeof(struct _msg))
  {
    if (eqstri(msg.from,usr.name) || GEPriv(usr.priv,SYSOP))
    {
      /* Now check the high-water mark, to see if the msg has been scanned  *
       * yet.                                                               */
      
      if (! (area.attrib[class] & ECHOMAIL))
        hwm=0;
      else
      {
        sprintf(temp,d_msg,1);

        if ((hwmf=shopen(temp,O_RDONLY | O_BINARY))==-1)
          hwm=0;
        else
        {
          read(hwmf,(char *)&hwmsg,sizeof(struct _msg));
          close(hwmf);

          hwm=max(hwmsg.reply,hwmsg.up);
        }
      }

      /* Also check the REC'D bit, and the SENT bit */
      
      if (((area.attrib[class] & SYSMAIL) && (msg.attr & MSGSENT)) ||
          (msg.attr & MSGREAD) || lmsg <= hwm)
      {
        Display_File(0,NULL,"%sCHG_SENT",PRM(misc_path));

        /* Only allow the SysOp to edit a message which has been sent/rec'd */
        
        if (! GEPriv(usr.priv,SYSOP))
        {
          close(msgfile);
          return -1;
        }
      }

      /* Strip off MSGREAD and MSGSENT bits, and add MSGLOCAL. */

      msg.attr &= ~(MSGREAD | MSGSENT);
      msg.attr |= MSGLOCAL;

      Add_Message_Date(&msg);

      m_zone=prm.address[0].Zone;
      m_net=msg.dest_net;
      m_node=msg.dest;
      m_point=0;

      Scan_For_Intl_And_Fmpt(msgfile,&m_zone,&m_net,&m_node,&m_point,
           s(reply_str),s(topt_str),1);

      if (area.attrib[class] & SYSMAIL)
        sprintf(netnode,znnp,m_zone,m_net,m_node,m_point);
      else mknl(netnode);

      if (Get_Msg_Attr(&msg)==-1)
        aborted=TRUE;
      else
      {
        logit(usingeditor);
        x=Editor(&msg,msgfile);
        logit(outofeditor);

        if (x != ABORT)
        {
          if (x==LOCAL_EDIT)
            aborted=FALSE;
          else aborted=Save_Msg(&msg,NULL,FALSE,lmsg,TRUE);
        }
        else aborted=TRUE;
      }

      if (aborted)
        Puts(msg_aborted);
    }
    else aborted=Msg_Change_Err();
  }
  else aborted=Msg_Change_Err();

  if (msgfile != -1)
    close(msgfile);

  if (aborted)
    return -1;
  else return 0;
}

static int pascal Msg_Change_Err(void)
{
  Display_File(0,NULL,"%sCHG_NO",PRM(misc_path));
  return TRUE;
}

void pascal Blank_Msg(struct _msg *msg)
{
  memset(msg,'\x00',sizeof(struct _msg));

  Add_Message_Date(msg);

  strcpy(msg->from,usr.name);
  msg->attr=MSGLOCAL;

  msg->dest=msg->orig=prm.address[0].Node;
  msg->orig_net=msg->dest_net=prm.address[0].Net;
}


void pascal Add_Message_Date(struct _msg *msg)
{
  time_t longtime;
  struct tm *localt;

  longtime=time(NULL);
  localt=localtime(&longtime);

  strftime(msg->date,20,"%d %b %y  %H:%M:%S",localt);

  localt=localtime(&longtime);

  msg->date_written.date.da=msg->date_arrived.date.da=localt->tm_mday;
  msg->date_written.date.mo=msg->date_arrived.date.mo=localt->tm_mon+1;
  msg->date_written.date.yr=msg->date_arrived.date.yr=localt->tm_year-80;

  msg->date_written.time.hh=msg->date_arrived.time.hh=localt->tm_hour;
  msg->date_written.time.mm=msg->date_arrived.time.mm=localt->tm_min;
  msg->date_written.time.ss=msg->date_arrived.time.ss=localt->tm_sec >> 1;
}


int pascal Msg_Reply(void)
{
  struct _msg msg,
              mmsg;

  char temp[PATHLEN],
       savebuf[PATHLEN];

  int msgfile,
      msgupsave,
      aborted,
      m_zone,
      m_net,
      m_node,
      m_point,
      x;

  strcpy(savebuf,linebuf);
  *linebuf='\0';
    
  sprintf(temp,d_msg,last_msg);

  if ((msgfile=shopen(temp,O_RDWR | O_BINARY)) != -1 &&
      read(msgfile,(char *)&mmsg,sizeof(struct _msg))==sizeof(struct _msg))
  {
    if (! CanSeeMsg(&mmsg))
    {
      close(msgfile);
      Msg_Enter();
      return -1;
    }

    m_zone=prm.address[0].Zone;
    m_net=mmsg.orig_net;
    m_node=mmsg.orig;
    m_point=0;

    Scan_For_Intl_And_Fmpt(msgfile,&m_zone,&m_net,&m_node,&m_point,s(msgid_str),s(fmpt_str),2);

    close(msgfile);

    isareply=TRUE;
    msgupsave=mmsg.up;

    Blank_Msg(&msg);

    /* If we're replying to a private message, make our message private    *
     * by default.  (However, if the area we're in doesn't allow private   *
     * messages, then this setting doesn't matter.)                        */

    if (mmsg.attr & MSGPRIVATE)
      msg.attr |= MSGPRIVATE;

    /* Fix the reply-chain */

    msg.reply=last_msg;
    msg.up=msgupsave;

    strcpy(msg.to,mmsg.from);

    if (toupper(mmsg.subj[0])=='R' && toupper(mmsg.subj[1])=='E' &&
        mmsg.subj[2]==':')
      mknl(msg.subj);
    else strcpy(msg.subj,"Re: ");

    strcat(msg.subj,mmsg.subj);

    if (area.attrib[class] & SYSMAIL)
      sprintf(netnode,znnp,m_zone,m_net,m_node,m_point);
    else mknl(netnode);

    if (Get_Msg_Attr(&msg)==-1)
      aborted=TRUE;
    else
    {
      logit(usingeditor);
      x=Editor(&msg,-1);
      logit(outofeditor);

      if (x != ABORT)
      {
        if (x==LOCAL_EDIT)
          aborted=FALSE;
        else aborted=Save_Msg(&msg,NULL,FALSE,0,0);

        /* Write original message again, to set the REPLY link correctly! */

        if (!aborted && (msgfile=shopen(temp,O_RDWR | O_BINARY)) != -1)
        {
          mmsg.up=high_msg;
          write(msgfile,(char *)&mmsg,sizeof(struct _msg));
          close(msgfile);
        }
      }
      else aborted=TRUE;
    }

    isareply=FALSE;

    if (aborted)
      Puts(msg_aborted);
    else
    {
/*    if (killwhendone)
        Msg_Kill(last_msg); */

      strcpy(linebuf,savebuf);    /* So that "rk" will work for reply/kill */

      if (eqstri(linebuf,"k"))
        strcat(linebuf,"=");
      
      return 0;
    }
  }
  else
  {
    if (msgfile != -1)
      close(msgfile);

    return (Msg_Enter());
  }

  return -1;
}
/* Dig any ^aINTL and ^aFMPT lines out of the message we're replying to */

void pascal Scan_For_Intl_And_Fmpt(int msgfile,int *m_zone,int *m_net,int *m_node,int *m_point,char *str_msgid,char *str_fmpt,int intl_num)
{
  int x;

  char bufr[FWD_BUFSIZ],
       temp[PATHLEN],
       temp2[PATHLEN],
       *p,
       *s;

  int zo,ne,no,pt;

  while ((x=read(msgfile,bufr,FWD_BUFSIZ)) > 0)
  {
    if ((p=memstr(bufr,str_msgid,x,6)) != NULL)
    {
      s=firstchar(p+7," :",1);
      strncpy(orig_msgid,s,50);
      orig_msgid[50]='\0';

      if (strchr(orig_msgid,'\r'))
        *strchr(orig_msgid,'\r')='\0';

      Parse_NetNode(orig_msgid,&zo,&ne,&no,&pt);

      /* Only grab info from the MSGID if it was from the same node as in  *
       * the message header...  Oterhwise, it could be from someone else   *
       * (as in a forwarded message), which isn't good.                    */

      if (ne==*m_net && no==*m_node)
      {
        *m_zone=zo;
        *m_point=pt;
      }
    }
    else mknl(orig_msgid);

    if ((area.attrib[class] & SYSMAIL) &&
        (p=memstr(bufr,str_fmpt,x,5)) != NULL)
      *m_point=atoi(p+6);

    if ((area.attrib[class] & SYSMAIL) &&
        (p=memstr(bufr,s(intl_str),x,5)) != NULL)
    {
      getword(p+6,temp,"\r",1);
      getword(temp,temp2," ",intl_num);

      Parse_NetNode(temp2,m_zone,m_net,m_node,NULL);
    }

    if (! eof(msgfile))
      lseek(msgfile,-(long)FWD_OVERLAP,SEEK_CUR);
    else break;
  }
}


int pascal Editor(struct _msg *msg,int msgfile)
{
  struct _css css;

  int ret;

  last_maxed=FALSE;

  ChatSaveStatus(&css);
  ChatSetStatus(FALSE,s(cs_enter_msg));

  Putc('\n');

  if (*PRM(local_editor) && (local || (GEPriv(usr.priv,ASSTSYSOP) &&
                                       *PRM(local_editor)=='!')))
    ret=Local_Editor(msg,msgfile);
  else if (usr.use_lore || (prm.flags & FLAG_no_magnet))
  {
    Putc('\n');
    ret=Bored(msg,msgfile);
  }
  else if (!usr.use_lore && (graph==GRAPHICS_none || usr.width < 79 ||
           usr.len < 24))
  {
    Puts(s(req_graph));
    ret=Bored(msg,msgfile);
  }
  else
  {
    Puts(CLS);

    inmagnet=last_maxed=TRUE;
    ret=MagnEt(msg,msgfile);
    inmagnet=FALSE;
  }

  ChatRestoreStatus(&css);

  return ret;
}


